from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class Consent(BaseModel):
    consent_id: Optional[str] = None
    field_name: Optional[str] = None
    field_id: Optional[str] = None
    display_text: Optional[str] = None
    error_message: Optional[str] = None
    type: Optional[str] = None
    pre_checked: Optional[bool] = None
    required: Optional[bool] = None
    enabled: Optional[bool] = None
    field_id_enabled: Optional[bool] = None


Consent.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.country_dto import CountryDto
from pianosdk.publisher.models.region_dto import RegionDto


class UserAddressApiDto(BaseModel):
    user_address_id: Optional[str] = None
    country: Optional['CountryDto'] = None
    region: Optional['RegionDto'] = None
    city: Optional[str] = None
    postal_code: Optional[str] = None
    company_name: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    address1: Optional[str] = None
    address2: Optional[str] = None
    phone: Optional[str] = None


UserAddressApiDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user_access_dto import UserAccessDto
from pianosdk.publisher.models.user_address_api_dto import UserAddressApiDto
from pianosdk.publisher.models.user_dto import UserDto


class GiftInfoDto(BaseModel):
    gift_id: Optional[str] = None
    status: Optional[str] = None
    redeem_code: Optional[str] = None
    email: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    welcome_message: Optional[str] = None
    created: Optional[datetime] = None
    redeemed: Optional[datetime] = None
    user: Optional['UserDto'] = None
    subscription_id: Optional[str] = None
    user_access: Optional['UserAccessDto'] = None
    user_address: Optional['UserAddressApiDto'] = None


GiftInfoDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class SubscriptionRestrictions(BaseModel):
    allow_change_next_bill_date: Optional[bool] = None
    allow_enable_auto_renew: Optional[bool] = None
    allow_switch_payment_method: Optional[bool] = None
    allow_scheduler_renewals: Optional[bool] = None
    allow_future_renewals: Optional[bool] = None
    allow_verify_now: Optional[bool] = None
    allow_activate_now: Optional[bool] = None


SubscriptionRestrictions.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UserSubscriptionDetailsDto(BaseModel):
    term_name: Optional[str] = None
    term_id: Optional[str] = None
    type: Optional[str] = None
    payment_billing_plan: Optional[str] = None
    billing_plan: Optional[str] = None
    subscription_id: Optional[str] = None
    is_stripe: Optional[bool] = None
    payment_method: Optional[str] = None
    status: Optional[str] = None
    status_display: Optional[str] = None
    user_payment_info_id: Optional[str] = None
    access_id: Optional[str] = None
    last_conversion_id: Optional[str] = None
    last_payment_conversion_id: Optional[str] = None
    delivery_schedule_pub_id: Optional[str] = None
    owner: Optional[str] = None
    stripe_billing_sync_state: Optional[str] = None
    stripe_subscription_id: Optional[str] = None


UserSubscriptionDetailsDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.import_section import ImportSection
from typing import List


class ImportData(BaseModel):
    import_data_id: Optional[str] = None
    import_data_status: Optional[str] = None
    sections: Optional['List[ImportSection]'] = None


ImportData.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class LinkedTermEventPageViewContent(BaseModel):
    content_created: Optional[str] = None
    content_author: Optional[str] = None
    content_section: Optional[str] = None
    content_type: Optional[str] = None
    tags: Optional[List[str]] = None


LinkedTermEventPageViewContent.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LightExperience(BaseModel):
    experience_id: Optional[str] = None
    title: Optional[str] = None
    aid: Optional[str] = None
    app_name: Optional[str] = None


LightExperience.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class WebhookStatus(BaseModel):
    status: Optional[str] = None
    description: Optional[str] = None


WebhookStatus.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class CheckoutConfigurationCheckoutFlowModel(BaseModel):
    name: Optional[str] = None
    description: Optional[str] = None
    checkout_flow_id: Optional[str] = None
    checkout_configuration_id: Optional[str] = None
    is_default: Optional[bool] = None


CheckoutConfigurationCheckoutFlowModel.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.currency import Currency


class Money(BaseModel):
    amount: Optional[float] = None
    currency: Optional['Currency'] = None


Money.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.anon.models.o_auth_token import OAuthToken


class OauthApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def auth_token(self, client_id: str, client_secret: str = None, code: str = None, refresh_token: str = None, grant_type: str = None, redirect_uri: str = None, username: str = None, password: str = None, state: str = None, device_id: str = None) -> ApiResponse[OAuthToken]:
        _url_path = '/api/v3/oauth/authToken'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'client_id': _encode_parameter(client_id),
            'client_secret': _encode_parameter(client_secret),
            'code': _encode_parameter(code),
            'refresh_token': _encode_parameter(refresh_token),
            'grant_type': _encode_parameter(grant_type),
            'redirect_uri': _encode_parameter(redirect_uri),
            'username': _encode_parameter(username),
            'password': _encode_parameter(password),
            'state': _encode_parameter(state),
            'device_id': _encode_parameter(device_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OAuthToken)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class AccessPeriod(BaseModel):
    period_id: Optional[str] = None
    name: Optional[str] = None
    billing_description: Optional[str] = None
    type: Optional[str] = None
    enabled: Optional[bool] = None
    removed: Optional[bool] = None
    tooltip: Optional[str] = None


AccessPeriod.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ExternalCss(BaseModel):
    title: Optional[str] = None
    url: Optional[str] = None
    status: Optional[str] = None
    position: Optional[int] = None
    external_css_id: Optional[str] = None


ExternalCss.model_rebuild()



from typing import Dict

from requests import session, Response
from requests.adapters import HTTPAdapter
from urllib3 import Retry

from .models import HttpRequest, HttpResponse


class RequestsClient:

    def __init__(self,
                 timeout=60,
                 max_retries=None,
                 backoff_factor=None,
                 verify=True):
        """The constructor.
        Args:
            timeout (float): The default global timeout(seconds).
        """
        self.timeout = timeout
        self.session = session()

        retries = Retry(total=max_retries, backoff_factor=backoff_factor)
        self.session.mount('http://', HTTPAdapter(max_retries=retries))
        self.session.mount('https://', HTTPAdapter(max_retries=retries))

        self.session.verify = verify

    def execute(self, request: HttpRequest) -> HttpResponse:
        """
        Execute a given HttpRequest to get a string response back
        :param request: The given HttpRequest to execute.
        :return: The response of the HttpRequest.
        """
        response = self.session.request(
            request.http_method,
            request.query_url,
            headers=request.headers,
            params=request.query_parameters,
            data=request.parameters,
            json=request.json,
            files=request.files,
            timeout=self.timeout
        )
        response.encoding = 'utf-8'

        return self._convert_response(response, request)

    @staticmethod
    def _convert_response(response: Response, http_request: HttpRequest) -> HttpResponse:
        return HttpResponse(
            response.status_code,
            response.reason,
            response.headers,
            response.text,
            http_request
        )

    @staticmethod
    def build_request(method: str,
                      query_url: str,
                      headers: Dict = None,
                      query_parameters: Dict = None,
                      parameters: Dict = None,
                      json: Dict = None,
                      files: Dict = None) -> HttpRequest:
        """
        Create a HttpRequest object for the given parameters
        :param method: The request method
        :param query_url: The URL to send the request to.
        :param headers: The headers for the HTTP Request.
        :param query_parameters: Query parameters to add in the URL.
        :param parameters: Form or body parameters to be included in the body.
        :param json: Dict to be sent with the request as body.
        :param files: Files to be sent with the request.
        :return: The generated HttpRequest for the given parameters.
        """
        return HttpRequest(method.upper(),
                           query_url,
                           headers,
                           query_parameters,
                           parameters,
                           json,
                           files)



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.user_address import UserAddress
from pianosdk.publisher.models.user_address_history import UserAddressHistory


class PublisherUserAddressApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def create_user_address(self, aid: str, uid: str, country_id: str, city: str, postal_code: str, address1: str, region_id: str = None, company_name: str = None, first_name: str = None, last_name: str = None, address2: str = None, phone: str = None, region_name: str = None) -> ApiResponse[UserAddress]:
        _url_path = '/api/v3/publisher/user/address/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'region_id': _encode_parameter(region_id),
            'country_id': _encode_parameter(country_id),
            'city': _encode_parameter(city),
            'postal_code': _encode_parameter(postal_code),
            'company_name': _encode_parameter(company_name),
            'first_name': _encode_parameter(first_name),
            'last_name': _encode_parameter(last_name),
            'address1': _encode_parameter(address1),
            'address2': _encode_parameter(address2),
            'phone': _encode_parameter(phone),
            'region_name': _encode_parameter(region_name)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, UserAddress)
        return _result

    def delete_user_address(self, aid: str, uid: str, user_address_id: str) -> ApiResponse[bool]:
        _url_path = '/api/v3/publisher/user/address/delete'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'user_address_id': _encode_parameter(user_address_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, bool)
        return _result

    def list_user_address(self, aid: str, uid: str, offset: int = 0, limit: int = 100) -> ApiResponse[List[UserAddress]]:
        _url_path = '/api/v3/publisher/user/address/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, UserAddress)
        return _result

    def list_user_address_history(self, aid: str, offset: int = 0, limit: int = 100, date_from: datetime = None, date_to: datetime = None) -> ApiResponse[List[UserAddressHistory]]:
        _url_path = '/api/v3/publisher/user/address/history'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'date_from': _encode_parameter(date_from),
            'date_to': _encode_parameter(date_to),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, UserAddressHistory)
        return _result

    def update_user_address(self, aid: str, uid: str, user_address_id: str, country_id: str, city: str, postal_code: str, address1: str, region_id: str = None, company_name: str = None, first_name: str = None, last_name: str = None, address2: str = None, phone: str = None, region_name: str = None, additional_fields: str = None) -> ApiResponse[UserAddress]:
        _url_path = '/api/v3/publisher/user/address/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'user_address_id': _encode_parameter(user_address_id),
            'region_id': _encode_parameter(region_id),
            'country_id': _encode_parameter(country_id),
            'city': _encode_parameter(city),
            'postal_code': _encode_parameter(postal_code),
            'company_name': _encode_parameter(company_name),
            'first_name': _encode_parameter(first_name),
            'last_name': _encode_parameter(last_name),
            'address1': _encode_parameter(address1),
            'address2': _encode_parameter(address2),
            'phone': _encode_parameter(phone),
            'region_name': _encode_parameter(region_name),
            'additional_fields': _encode_parameter(additional_fields)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, UserAddress)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.promotion_fixed_discount import PromotionFixedDiscount
from typing import List


class Promotion(BaseModel):
    promotion_id: Optional[str] = None
    aid: Optional[str] = None
    name: Optional[str] = None
    status: Optional[str] = None
    fixed_promotion_code: Optional[str] = None
    unlimited_uses: Optional[bool] = None
    promotion_code_prefix: Optional[str] = None
    new_customers_only: Optional[bool] = None
    discount_amount: Optional[float] = None
    discount_currency: Optional[str] = None
    discount: Optional[str] = None
    percentage_discount: Optional[float] = None
    discount_type: Optional[str] = None
    uses_allowed: Optional[int] = None
    uses: Optional[int] = None
    never_allow_zero: Optional[bool] = None
    term_dependency_type: Optional[str] = None
    start_date: Optional[datetime] = None
    end_date: Optional[datetime] = None
    create_date: Optional[datetime] = None
    create_by: Optional[str] = None
    update_date: Optional[datetime] = None
    update_by: Optional[str] = None
    deleted: Optional[bool] = None
    fixed_discount_list: Optional['List[PromotionFixedDiscount]'] = None
    apply_to_all_billing_periods: Optional[bool] = None
    can_be_applied_on_renewal: Optional[bool] = None
    billing_period_limit: Optional[int] = None


Promotion.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class PublisherSetPasswordRequest(BaseModel):
    aid: Optional[str] = None
    reset_password_token: Optional[str] = None
    password: Optional[str] = None


PublisherSetPasswordRequest.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.linked_term_payment_params import LinkedTermPaymentParams


class LinkedTermConversionParams(BaseModel):
    conversion_id: Optional[str] = None
    create_date: Optional[int] = None
    payment: Optional['LinkedTermPaymentParams'] = None


LinkedTermConversionParams.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.composer import Composer
from pianosdk.publisher.models.my_account import MyAccount
from pianosdk.publisher.models.redemption_page import RedemptionPage
from pianosdk.publisher.models.subscription_restrictions import SubscriptionRestrictions


class AppFeatures(BaseModel):
    my_account: Optional['MyAccount'] = None
    composer: Optional['Composer'] = None
    subscription_restrictions: Optional['SubscriptionRestrictions'] = None
    redemption_page: Optional['RedemptionPage'] = None
    is_payment_mock_enabled: Optional[bool] = None
    is_publisher_dashboard_localization_enabled: Optional[bool] = None
    is_checkout_authentication_in_separate_state: Optional[bool] = None


AppFeatures.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user import User


class OfferTemplateRevision(BaseModel):
    offer_template_id: Optional[str] = None
    token_type: Optional[str] = None
    aid: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    deleted: Optional[bool] = None
    create_date: Optional[datetime] = None
    create_by: Optional['User'] = None
    update_date: Optional[datetime] = None
    update_by: Optional['User'] = None
    publish_date: Optional[datetime] = None
    version: Optional[int] = None
    type: Optional[str] = None
    type_id: Optional[str] = None
    is_published: Optional[bool] = None
    content_loaded: Optional[bool] = None
    content1_type: Optional[str] = None
    content2_type: Optional[str] = None
    content1_value: Optional[str] = None
    content2_value: Optional[str] = None


OfferTemplateRevision.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class EigenCreateCreditCardInfoDto(BaseModel):
    integration_version: Optional[str] = None


EigenCreateCreditCardInfoDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class PlatformBillingConfigurationDTO(BaseModel):
    platform_billing_configuration_id: Optional[str] = None
    aid: Optional[str] = None
    account_id: Optional[str] = None
    properties: Optional[str] = None


PlatformBillingConfigurationDTO.model_rebuild()



from copy import deepcopy
from typing import Dict


class ApiException(Exception):
    request_id: str
    error_code: int
    errors: Dict[str, str]
    is_transport_error: bool

    def __init__(self, request_id: str, message: str, code: int = 0, errors: Dict[str, str] = None, is_transport_error: bool = False, *args: object) -> None:
        super().__init__(message, *args)
        self.request_id = request_id
        self.error_code = code
        self.errors = errors and deepcopy(errors)
        self.is_transport_error = is_transport_error



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.country import Country
from pianosdk.publisher.models.delivery_zone import DeliveryZone
from pianosdk.publisher.models.external_api_field import ExternalAPIField
from pianosdk.publisher.models.resource import Resource
from pianosdk.publisher.models.schedule import Schedule
from pianosdk.publisher.models.term_change_option import TermChangeOption
from pianosdk.publisher.models.vouchering_policy import VoucheringPolicy
from typing import Any
from typing import Dict
from typing import List


class Term(BaseModel):
    term_id: Optional[str] = None
    aid: Optional[str] = None
    resource: Optional['Resource'] = None
    type: Optional[str] = None
    type_name: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    product_category: Optional[str] = None
    verify_on_renewal: Optional[bool] = None
    create_date: Optional[datetime] = None
    update_date: Optional[datetime] = None
    term_billing_descriptor: Optional[str] = None
    payment_billing_plan: Optional[str] = None
    payment_billing_plan_description: Optional[str] = None
    payment_billing_plan_table: Optional['List[Any]'] = None
    payment_allow_renew_days: Optional[int] = None
    payment_force_auto_renew: Optional[bool] = None
    payment_is_custom_price_available: Optional[bool] = None
    payment_is_subscription: Optional[bool] = None
    payment_has_free_trial: Optional[bool] = None
    payment_new_customers_only: Optional[bool] = None
    payment_trial_new_customers_only: Optional[bool] = None
    payment_allow_promo_codes: Optional[bool] = None
    payment_renew_grace_period: Optional[int] = None
    payment_allow_gift: Optional[bool] = None
    payment_currency: Optional[str] = None
    currency_symbol: Optional[str] = None
    payment_first_price: Optional[float] = None
    schedule: Optional['Schedule'] = None
    schedule_billing: Optional[str] = None
    custom_require_user: Optional[bool] = None
    custom_default_access_period: Optional[int] = None
    adview_vast_url: Optional[str] = None
    adview_access_period: Optional[int] = None
    registration_access_period: Optional[int] = None
    registration_grace_period: Optional[int] = None
    external_api_id: Optional[str] = None
    external_api_name: Optional[str] = None
    external_api_source: Optional[int] = None
    external_api_form_fields: Optional['List[ExternalAPIField]'] = None
    evt_verification_period: Optional[int] = None
    evt_fixed_time_access_period: Optional[int] = None
    evt_grace_period: Optional[int] = None
    evt_itunes_bundle_id: Optional[str] = None
    evt_itunes_product_id: Optional[str] = None
    evt_google_play_product_id: Optional[str] = None
    evt_cds_product_id: Optional[str] = None
    evt: Optional['Term'] = None
    collect_address: Optional[bool] = None
    delivery_zone: Optional['List[DeliveryZone]'] = None
    default_country: Optional['Country'] = None
    vouchering_policy: Optional['VoucheringPolicy'] = None
    billing_config: Optional[str] = None
    is_allowed_to_change_schedule_period_in_past: Optional[bool] = None
    collect_shipping_address: Optional[bool] = None
    change_options: Optional['List[TermChangeOption]'] = None
    shared_account_count: Optional[int] = None
    shared_redemption_url: Optional[str] = None
    billing_configuration: Optional[str] = None
    show_full_billing_plan: Optional[bool] = None
    external_term_id: Optional[str] = None
    external_product_ids: Optional[str] = None
    subscription_management_url: Optional[str] = None
    custom_data: Optional['Dict[str, Any]'] = None
    allow_start_in_future: Optional[bool] = None
    maximum_days_in_advance: Optional[int] = None
    allow_renewable_gifting: Optional[bool] = None
    gift_redemption_url: Optional[str] = None


Term.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.term import Term
from pianosdk.publisher.models.upgrade_option import UpgradeOption
from pianosdk.publisher.models.user import User
from typing import List


class BaseOfferModel(BaseModel):
    offer_id: Optional[str] = None
    name: Optional[str] = None
    type: Optional[str] = None
    create_date: Optional[datetime] = None
    update_date: Optional[datetime] = None
    create_by: Optional['User'] = None
    update_by: Optional['User'] = None
    terms: Optional['List[Term]'] = None
    upgrade_options: Optional['List[UpgradeOption]'] = None


BaseOfferModel.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.deployment_content_fields import DeploymentContentFields
from typing import List


class DeploymentVariantParamDto(BaseModel):
    offer_template_variant_id: Optional[str] = None
    name: Optional[str] = None
    content_field_list: Optional['List[DeploymentContentFields]'] = None


DeploymentVariantParamDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class Email(BaseModel):
    email_id: Optional[int] = None
    name: Optional[str] = None
    caption: Optional[str] = None
    description: Optional[str] = None
    publisher_config: Optional[str] = None
    default_template_id: Optional[str] = None


Email.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.source_term_change_variant import SourceTermChangeVariant
from typing import List


class SourceTermChangeVariants(BaseModel):
    term_and_periods: Optional['List[SourceTermChangeVariant]'] = None


SourceTermChangeVariants.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class CountryDto(BaseModel):
    country_id: Optional[str] = None
    country_name: Optional[str] = None
    country_code: Optional[str] = None


CountryDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ErrorCode(BaseModel):
    code: Optional[int] = None
    message: Optional[str] = None


ErrorCode.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ResourceStats(BaseModel):
    rid: Optional[str] = None
    num_bundles: Optional[int] = None
    num_customers: Optional[int] = None
    num_terms: Optional[int] = None
    tags: Optional[str] = None


ResourceStats.model_rebuild()



import base64
import hashlib
import hmac
import json
from datetime import datetime, timezone
from typing import TypeVar, Dict, List, Union, Type, Callable, Generic, Any, Optional, overload

from Crypto.Cipher import AES
from pydantic import BaseModel

from pianosdk.api_response import ApiResponse
from pianosdk.exceptions import ApiException
from pianosdk.httpwrap import HttpResponse

_T = TypeVar('_T')
_S = TypeVar('_S')
_ObjOrList = Union[_T, List[_T]]
_DELIMITER = '~~~'


def _encrypt(key: str, value: str) -> str:
    value_bytes = value.encode()
    padding_length = (16 - len(value_bytes)) % 16
    value_bytes += bytes([padding_length]) * padding_length
    aes = _get_aes_for_key(key)
    out = aes.encrypt(value_bytes)
    safe = _urlsafe_encode_with_strip(out)
    return safe + _DELIMITER + _urlsafe_encode_with_strip(
        hmac.new(key.encode(), safe.encode(), digestmod=hashlib.sha256).digest())


def _decrypt(key: str, value: str) -> str:
    data, *hmac_value = value.split(_DELIMITER)
    if len(hmac_value) > 1:
        raise ValueError("Invalid message. We've found data after HMAC")
    if hmac_value:
        test_hmac = _urlsafe_encode_with_strip(hmac.new(key.encode(), data.encode(), digestmod=hashlib.sha256).digest())
        if test_hmac != hmac_value[0]:
            raise ValueError('Could not parse message: invalid HMAC')
    return _decrypt_to_bytes(key, data)


def _decrypt_to_bytes(key: str, value: str) -> str:
    data = _urlsafe_decode_stripped(value)
    aes = _get_aes_for_key(key)
    out = aes.decrypt(data)
    return _unpad(out).decode()


def _unpad(data: bytes) -> bytes:
    padding = data[-1]
    return data[:-padding] if 0 < padding <= 16 else data


def _get_aes_for_key(key: str):
    cipher_key = key[:32] if len(key) > 32 else key.ljust(32, 'X')
    return AES.new(key=cipher_key.encode(), mode=AES.MODE_ECB)


def _urlsafe_encode_with_strip(data: bytes) -> str:
    return base64.urlsafe_b64encode(data).decode().rstrip('=')


def _urlsafe_decode_stripped(data: str) -> bytes:
    return base64.urlsafe_b64decode(data + '=' * ((4 - len(data)) % 4))


def _json_deserialize(response: HttpResponse, cls: Type[_T] = None) -> ApiResponse[_ObjOrList[_T]]:
    if response.status_code >= 300:
        raise ApiException(
            response.request_id,
            f'HTTP Error: {response.status_code} {response.reason_phrase}',
            response.status_code,
            {},
            True
        )
    request_id = response.request_id or ''
    json_data = json.loads(response.text)
    code = json_data.get('code', response.status_code)
    if code != 0:
        raise ApiException(
            request_id,
            json_data.get('message') or response.reason_phrase,
            code,
            json_data.get('validation_errors') or json_data.get('error_code_list', {})
        )

    const_keys = {'code', 'count', 'limit', 'offset', 'total', 'ts'}
    data = {k: v for k, v in json_data.items() if k in const_keys}
    data_keys = json_data.keys() - const_keys
    if len(data_keys) > 1:
        raise ApiException(request_id, f"SDK doesn't support this endpoint because it has different format [{data_keys}]")
    obj_data = json_data.get(next(iter(data_keys), None))
    if obj_data is None:
        obj_data = {}
    if cls is not None and cls == Any:
        cls = None
    data['data'] = obj_data if cls is None or not issubclass(cls, BaseModel) else _unbox_object(obj_data, cls)
    return ApiResponse(request_id, **data)


def _unbox_object(data: Union[Dict, List], cls: Type[_T]) -> _ObjOrList[_T]:
    many = type(data) == list
    return cls(**data) if not many else [cls(**x) for x in data]


def _encode_parameter(value: Optional[Any]) -> Optional[Any]:
    if value is None:
        return None
    if isinstance(value, datetime):
        return int(value.replace(tzinfo=timezone.utc).timestamp())
    return value


class cached_property(Generic[_T]):
    func: Callable[[Any], _T]

    def __init__(self, func: Callable[[Any], _T]) -> None:
        self.__annotations__ = getattr(func, '__annotations__')
        self.__doc__ = getattr(func, '__doc__')
        self.__isabstractmethod__ = getattr(func, '__isabstractmethod__', False)
        self.func = func

    @overload
    def __get__(self, instance: None, owner: Optional[Type[Any]] = ...) -> 'cached_property[_T]': ...

    @overload
    def __get__(self, instance: _S, owner: Optional[Type[Any]] = ...) -> _T: ...

    def __get__(self, instance, owner):
        if instance is None:
            return self
        value = instance.__dict__[self.func.__name__] = self.func(instance)
        return value

    def __repr__(self) -> str:
        return f'<{self.__class__.__name__} func={self.func}>'



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.app import App


class PaypalExpressCheckoutConfiguration(BaseModel):
    configuration_id: Optional[str] = None
    app: Optional['App'] = None
    source_name: Optional[str] = None
    source_id: Optional[int] = None
    title: Optional[str] = None
    is_editable: Optional[bool] = None
    is_disabled: Optional[bool] = None
    properties: Optional[str] = None
    is_visible: Optional[bool] = None


PaypalExpressCheckoutConfiguration.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import Dict
from typing import List


class PublisherRegisterRequest(BaseModel):
    aid: Optional[str] = None
    email: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    password: Optional[str] = None
    consents: Optional[str] = None
    custom_fields: Optional[str] = None
    form_id: Optional[str] = None
    phone: Optional[str] = None
    aliases: Optional[Dict[str, str]] = None
    phone_digital_code: Optional[str] = None
    confirmed_email: Optional[bool] = None
    magic_link_sent: Optional[bool] = None
    passwordless: Optional[bool] = None


PublisherRegisterRequest.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.anon.models.resource import Resource


class Term(BaseModel):
    term_id: Optional[str] = None
    aid: Optional[str] = None
    resource: Optional['Resource'] = None
    type: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    create_date: Optional[datetime] = None


Term.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class RenderedEmail(BaseModel):
    body: Optional[str] = None
    subject: Optional[str] = None


RenderedEmail.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.period_reference_dto import PeriodReferenceDTO
from typing import List


class DeliveryScheduleConstraintViolationDTO(BaseModel):
    conflicting_periods_references: Optional['List[PeriodReferenceDTO]'] = None
    target_period_reference: Optional['PeriodReferenceDTO'] = None
    message: Optional[str] = None
    constraint_violation_code: Optional[str] = None


DeliveryScheduleConstraintViolationDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LinkedTermChurnParams(BaseModel):
    type: Optional[str] = None
    amount: Optional[float] = None
    currency: Optional[str] = None


LinkedTermChurnParams.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TranslationMapRevision(BaseModel):
    version: Optional[int] = None
    create_date: Optional[datetime] = None
    update_date: Optional[datetime] = None
    published: Optional[bool] = None
    editing: Optional[bool] = None
    draft: Optional[bool] = None


TranslationMapRevision.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.access import Access
from pianosdk.publisher.models.period import Period
from pianosdk.publisher.models.promo_code import PromoCode
from pianosdk.publisher.models.schedule import Schedule
from pianosdk.publisher.models.term import Term
from pianosdk.publisher.models.term_conversion_subscription import TermConversionSubscription
from pianosdk.publisher.models.user_payment_dto import UserPaymentDTO
from pianosdk.publisher.models.user_payment_info import UserPaymentInfo


class TermConversionDTO(BaseModel):
    term_conversion_id: Optional[str] = None
    term: Optional['Term'] = None
    type: Optional[str] = None
    aid: Optional[str] = None
    user_access: Optional['Access'] = None
    user_payment: Optional['UserPaymentDTO'] = None
    create_date: Optional[datetime] = None
    browser_id: Optional[str] = None
    subscription: Optional['TermConversionSubscription'] = None
    promo_code: Optional['PromoCode'] = None
    user_payment_info: Optional['UserPaymentInfo'] = None
    schedule: Optional['Schedule'] = None
    period: Optional['Period'] = None


TermConversionDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LinkedTermEventSessionOffer(BaseModel):
    offer_id: Optional[str] = None
    offer_template_id: Optional[str] = None
    offer_template_version_id: Optional[str] = None


LinkedTermEventSessionOffer.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user_address_dto import UserAddressDto


class TernChangeConfirmationDto(BaseModel):
    from_term_name: Optional[str] = None
    to_term_name: Optional[str] = None
    end_date: Optional[datetime] = None
    access_start_date: Optional[datetime] = None
    from_resource_name: Optional[str] = None
    to_resource_name: Optional[str] = None
    from_billing_plan: Optional[str] = None
    to_billing_plan: Optional[str] = None
    next_bill_date: Optional[datetime] = None
    prorate_refund_amount: Optional[str] = None
    prorate_amount: Optional[float] = None
    prorate_amount_display: Optional[str] = None
    prorate_amount_with_taxes: Optional[str] = None
    payment_method: Optional[str] = None
    user_address: Optional['UserAddressDto'] = None
    immediate_access: Optional[bool] = None
    immediate_billing: Optional[bool] = None
    to_term_amount: Optional[float] = None
    to_term_amount_display: Optional[str] = None
    to_term_amount_with_taxes: Optional[str] = None
    upi_expiration_month: Optional[int] = None
    upi_expiration_year: Optional[int] = None
    shared_account_limit: Optional[int] = None
    shared_account_count: Optional[int] = None
    prorate_access: Optional[bool] = None
    prorate_unused_amount: Optional[str] = None
    billing_timing_changed: Optional[bool] = None


TernChangeConfirmationDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.app import App
from pianosdk.publisher.models.inquiry_comment import InquiryComment
from pianosdk.publisher.models.resource import Resource
from pianosdk.publisher.models.user import User
from typing import List


class PaymentInquiry(BaseModel):
    payment_inquiry_id: Optional[str] = None
    resource: Optional['Resource'] = None
    app: Optional['App'] = None
    state: Optional[int] = None
    inquiry_reason: Optional[str] = None
    create_date: Optional[str] = None
    inquiry_comments: Optional['List[InquiryComment]'] = None
    category: Optional[str] = None
    update_state_by: Optional['User'] = None
    update_state_date: Optional[str] = None
    start_date: Optional[str] = None
    expire_date: Optional[str] = None
    transaction_date: Optional[str] = None
    transaction_id: Optional[str] = None
    spent_money: Optional[float] = None
    spent_money_display: Optional[str] = None
    source: Optional[str] = None
    currency: Optional[str] = None
    refunded_date: Optional[str] = None
    is_access_expired: Optional[bool] = None
    is_access_revoked: Optional[bool] = None
    is_access_unlimited: Optional[bool] = None
    refund_amount: Optional[str] = None
    refund_amount_recalculated: Optional[bool] = None


PaymentInquiry.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ChurnPreventionLogic(BaseModel):
    logic_id: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    create_date: Optional[datetime] = None
    update_date: Optional[datetime] = None
    default: Optional[bool] = None
    terms_count: Optional[int] = None
    scenarios: Optional[str] = None
    early_billing: Optional[str] = None


ChurnPreventionLogic.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TransactionItem(BaseModel):
    user_payment_id: Optional[str] = None
    name: Optional[str] = None
    payment_billing_plan: Optional[str] = None
    amount: Optional[float] = None
    refund_amount: Optional[float] = None
    payment_method: Optional[str] = None
    user_payment_info_id: Optional[str] = None
    payment_method_type: Optional[str] = None
    status: Optional[str] = None
    status_value: Optional[int] = None
    resource_image_url: Optional[str] = None
    resource_name: Optional[str] = None
    rid: Optional[str] = None
    currency_code: Optional[str] = None
    customer: Optional[str] = None
    _date: Optional[datetime] = None
    external_id: Optional[str] = None
    refund_external_tx_id: Optional[str] = None
    uid: Optional[str] = None
    term_id: Optional[str] = None
    price: Optional[float] = None
    price_display: Optional[str] = None
    currency: Optional[str] = None
    expires: Optional[int] = None
    taxed_price: Optional[str] = None
    upi_ext_customer_id: Optional[str] = None
    upi_ext_customer_id_label: Optional[str] = None
    transaction_type: Optional[str] = None
    currency_symbol: Optional[str] = None


TransactionItem.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.deployment_app import DeploymentApp
from pianosdk.publisher.models.user import User
from typing import List


class DeploymentListItem(BaseModel):
    deployment_id: Optional[str] = None
    global_template_version: Optional[int] = None
    deployment_date: Optional[datetime] = None
    user: Optional['User'] = None
    status: Optional[str] = None
    applications: Optional['List[DeploymentApp]'] = None


DeploymentListItem.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class CreditGuardStoredFields(BaseModel):
    terminal_number: Optional[str] = None
    card_id: Optional[str] = None
    card_mask: Optional[str] = None
    card_expiration: Optional[str] = None
    card_acquirer: Optional[str] = None
    auth_number: Optional[str] = None
    slave_terminal_number: Optional[str] = None
    slave_terminal_sequence: Optional[str] = None
    shovar: Optional[str] = None
    cg_uid: Optional[str] = None
    user_data_1: Optional[str] = None
    user_data_2: Optional[str] = None
    user_data_3: Optional[str] = None
    user_data_4: Optional[str] = None
    user_data_5: Optional[str] = None
    user_data_6: Optional[str] = None
    user_data_7: Optional[str] = None
    user_data_8: Optional[str] = None
    user_data_9: Optional[str] = None
    user_data_10: Optional[str] = None


CreditGuardStoredFields.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UserPaymentInfo(BaseModel):
    user_payment_info_id: Optional[str] = None
    description: Optional[str] = None
    upi_nickname: Optional[str] = None
    upi_number: Optional[str] = None
    upi_expiration_month: Optional[int] = None
    upi_expiration_year: Optional[int] = None
    upi_postal_code: Optional[str] = None
    upi_identifier: Optional[str] = None
    payment_method: Optional[str] = None
    payment_type: Optional[str] = None
    issuer_country_code: Optional[str] = None
    is_mock: Optional[bool] = None


UserPaymentInfo.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class CustomFieldAttributeDto(BaseModel):
    date_format: Optional[str] = None
    autofill: Optional[bool] = None
    autofill_type: Optional[str] = None
    default: Optional[bool] = None
    dmp_segmentation_enable: Optional[bool] = None
    multiline: Optional[bool] = None
    default_value: Optional[str] = None
    placeholder: Optional[str] = None
    pre_select_country_by_ip: Optional[bool] = None
    _global: Optional[bool] = None
    global_status: Optional[str] = None
    aid_list: Optional[List[str]] = None
    linked_term_field: Optional[bool] = None


CustomFieldAttributeDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TermInfo(BaseModel):
    term_id: Optional[str] = None
    period_id: Optional[str] = None
    title: Optional[str] = None
    billing: Optional[str] = None


TermInfo.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.user.models.delivery_schedule_constraint_violations_dto import DeliveryScheduleConstraintViolationsDTO


class UpdateDeliveryScheduleResponse(BaseModel):
    constraint_violations: Optional['DeliveryScheduleConstraintViolationsDTO'] = None


UpdateDeliveryScheduleResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class DynamicSubscriptionAccessPeriodInfo(BaseModel):
    access_period_id: Optional[str] = None
    name: Optional[str] = None
    total_iterations: Optional[int] = None
    start_date: Optional[datetime] = None
    end_date: Optional[datetime] = None
    iteration: Optional[int] = None
    payment_billing_plan: Optional[str] = None
    length: Optional[str] = None
    is_imported_access_period: Optional[bool] = None
    status: Optional[str] = None
    next_bill_date: Optional[datetime] = None


DynamicSubscriptionAccessPeriodInfo.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class LicenseeNotificationRule(BaseModel):
    notification_rule_id: Optional[str] = None
    licensee_id: Optional[str] = None
    contract_id_list: Optional[List[str]] = None
    parameter: Optional[str] = None
    condition: Optional[str] = None
    condition_value: Optional[int] = None
    is_for_all_contracts: Optional[bool] = None


LicenseeNotificationRule.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.term_period_reallocation import TermPeriodReallocation
from typing import List


class TermReallocation(BaseModel):
    name: Optional[str] = None
    term_type: Optional[str] = None
    term_pub_id: Optional[str] = None
    periods: Optional['List[TermPeriodReallocation]'] = None
    active_period_count: Optional[int] = None
    active_subscribers_count: Optional[int] = None


TermReallocation.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class PermissionDTO(BaseModel):
    title: Optional[str] = None
    descr: Optional[str] = None
    mnemonic: Optional[str] = None
    defaults: Optional[List[str]] = None


PermissionDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class OfferTemplateSubHistory(BaseModel):
    history_content: Optional[str] = None
    offer_template_history_event: Optional[str] = None
    offer_template_id: Optional[str] = None
    offer_template_variant_id: Optional[str] = None
    offer_template_version_id: Optional[str] = None


OfferTemplateSubHistory.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.webhook_config import WebhookConfig
from typing import List


class WebhookSettings(BaseModel):
    url: Optional[str] = None
    enabled: Optional[bool] = None
    configs: Optional['List[WebhookConfig]'] = None


WebhookSettings.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ChurnPreventionTemplateVersion(BaseModel):
    template_id: Optional[str] = None
    version_id: Optional[int] = None
    version: Optional[int] = None
    name: Optional[str] = None


ChurnPreventionTemplateVersion.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.resource import Resource
from pianosdk.publisher.models.resource_tag import ResourceTag


class PublisherResourceTagApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def attach_tag(self, resource_tag_id: str, aid: str, rid: str) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/resource/tag/attach'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'resource_tag_id': _encode_parameter(resource_tag_id),
            'aid': _encode_parameter(aid),
            'rid': _encode_parameter(rid)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def create_tag(self, aid: str, rid: str, name: str, type: str = 'standard') -> ApiResponse[ResourceTag]:
        _url_path = '/api/v3/publisher/resource/tag/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'rid': _encode_parameter(rid),
            'type': _encode_parameter(type),
            'name': _encode_parameter(name)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, ResourceTag)
        return _result

    def delete_tag(self, resource_tag_id: str, aid: str) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/resource/tag/delete'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'resource_tag_id': _encode_parameter(resource_tag_id),
            'aid': _encode_parameter(aid)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def detach_tag(self, resource_tag_id: str, aid: str, rid: str) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/resource/tag/detach'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'resource_tag_id': _encode_parameter(resource_tag_id),
            'aid': _encode_parameter(aid),
            'rid': _encode_parameter(rid)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def get_tag(self, resource_tag_id: str, aid: str) -> ApiResponse[ResourceTag]:
        _url_path = '/api/v3/publisher/resource/tag/get'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'resource_tag_id': _encode_parameter(resource_tag_id),
            'aid': _encode_parameter(aid)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, ResourceTag)
        return _result

    def list_bundles_for_tags(self, aid: str, included_tag_id: List[str], offset: int = 0, limit: int = 25, type: str = 'NA', order_direction: str = 'asc', order_by: str = 'name', q: str = None, disabled: bool = False, bundle_type: int = None) -> ApiResponse[List[Resource]]:
        _url_path = '/api/v3/publisher/resource/tag/bundles'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'included_tag_id': _encode_parameter(included_tag_id),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'disabled': _encode_parameter(disabled),
            'type': _encode_parameter(type),
            'bundle_type': _encode_parameter(bundle_type)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Resource)
        return _result

    def list_tags(self, aid: str, tag_type: int, offset: int = 0, limit: int = 25, rid: str = None, q: str = None, order_by: str = None, order_direction: str = None) -> ApiResponse[List[ResourceTag]]:
        _url_path = '/api/v3/publisher/resource/tag/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'rid': _encode_parameter(rid),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'tag_type': _encode_parameter(tag_type)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, ResourceTag)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.offer_template_history import OfferTemplateHistory
from typing import List


class OfferTemplateHistories(BaseModel):
    history_list: Optional['List[OfferTemplateHistory]'] = None


OfferTemplateHistories.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TermToIneligible(BaseModel):
    type: Optional[str] = None
    to_term_id: Optional[str] = None
    to_term_name: Optional[str] = None
    to_resource_id: Optional[str] = None
    to_resource_name: Optional[str] = None
    collect_address: Optional[bool] = None
    shared_account_count: Optional[int] = None
    tooltip: Optional[str] = None


TermToIneligible.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class GenerateCaptureContextParams(BaseModel):
    aid: Optional[str] = None
    target_origins: Optional[List[str]] = None
    source_id: Optional[int] = None


GenerateCaptureContextParams.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.term_info import TermInfo
from typing import List


class UpgradeOption(BaseModel):
    id: Optional[str] = None
    term_from: Optional['TermInfo'] = None
    term_to: Optional['TermInfo'] = None
    bill_timing: Optional[str] = None
    bill_timing_code: Optional[str] = None
    prorate_access: Optional[bool] = None
    immediate_access: Optional[bool] = None
    description: Optional[str] = None
    show_option_in_channels: Optional[List[str]] = None
    upgrade_offers: Optional[List[str]] = None


UpgradeOption.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LinkedTermPurchaseParams(BaseModel):
    trial: Optional[str] = None


LinkedTermPurchaseParams.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TemplateCounterModel(BaseModel):
    template_count: Optional[str] = None
    category: Optional[str] = None
    category_name_localized: Optional[str] = None


TemplateCounterModel.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.resource import Resource
from typing import List


class AddressUsageInfo(BaseModel):
    is_locked: Optional[bool] = None
    resources: Optional['List[Resource]'] = None
    has_usages: Optional[bool] = None


AddressUsageInfo.model_rebuild()
